/** 
 *  Hyper Operating System V4 Advance
 *
 * @file  ref_sem.c
 * @brief %jp{Z}tH̏ԎQ}%en{Reference Semaphore State}
 *
 * Copyright (C) 1998-2006 by Project HOS
 * http://sourceforge.jp/projects/hos/
 */



#include "core/core.h"
#include "object/semobj.h"



#if _KERNEL_SPT_REF_SEM


/** %jp{Z}tH̏ԎQ}%en{Reference Semaphore State}
 * @param  semid    %jp{lΏۂ̃Z}tHIDԍ}%en{ID number of the semaphore from which resource is acquired}
 * @param  pk_rsem  %jp{Z}tHԂԂpPbgւ̃|C^}%en{Pointer to the packet returning the semaphore state}
 * @retval E_OK     %jp{I}%en{Normal completion}
 * @retval E_ID     %jp{sIDԍ(semids邢͎gpłȂ)}%en{Invalid ID number(semid is invalid or unusable)}
 * @retval E_NOEXS  %jp{IuWFNg(ΏۃZ}tHo^)}%en{Non-existant object(specified semaphore is not registerd)}
 * @retval E_PAR    %jp{p[^G[(pk_rsems)}%en{Parameter error(pk_rsem is invalid)}
 */
ER ref_sem(ID semid, T_RSEM *pk_rsem)
{
	_KERNEL_T_SEMCB_PTR semcb;
	
	/* %jp{ID ̃`FbN} */
#if _KERNEL_SPT_REF_SEM_E_ID
	if ( !_KERNEL_SEM_CHECK_SEMID(semid) )
	{
		return E_ID;	/* %jp{sIDԍ}%en{Invalid ID number} */
	}
#endif
	
#if _KERNEL_SPT_REF_SEM_E_PAR
	if ( pk_rsem == NULL )
	{
		return E_PAR;	/* %jp{p[^G[}%en{Parameter error} */
	}
#endif
		
	_KERNEL_ENTER_SVC();		/* %jp{T[rXR[ɓ}%en{enter service-call} */
	
	/* %jp{IuWFNg݃`FbN} */
#if _KERNEL_SPT_REF_SEM_E_NOEXS
	if ( !_KERNEL_SEM_CHECK_EXS(semid) )
	{
		_KERNEL_LEAVE_SVC();	/* %jp{T[rXR[o}%en{leave service-call} */
		return E_NOEXS;			/* %jp{IuWFNg}%en{Non-existant object} */
	}
#endif

	/* %jp{Z}tHRg[ubN擾} */
	semcb = _KERNEL_SEM_ID2SEMCB(semid);
	
	/* %jp{擾} */
	pk_rsem->wtskid = _KERNEL_TSK_GET_TSKID(_KERNEL_REF_QUE(_KERNEL_SEM_GET_QUE(semcb)));
	pk_rsem->semcnt = _KERNEL_SEM_GET_SEMCNT(semcb);
	
	_KERNEL_LEAVE_SVC();	/* %jp{IuWFNg}%en{Non-existant object} */
	
	return E_OK;
}


#else	/* _KERNEL_SPT_REF_SEM */


#if _KERNEL_SPT_REF_SEM_E_NOSPT

/** %jp{Z}tH̊l(|[O)}%en{Acquire Semaphore Resource(Polling)}
 * @param  semid    %jp{lΏۂ̃Z}tHIDԍ}%en{ID number of the semaphore from which resource is acquired}
 * @retval E_NOSPT  %jp{T|[g@\}%en{Unsupported function}
 */
ER ref_sem(ID semid, T_RSEM *pk_rsem)
{
	return E_NOSPT;
}

#endif


#endif	/* _KERNEL_SPT_REF_SEM */



/* end of file */
